import React from 'react';
import './treeEditor.css';
import NodeMap from './interfaces/NodeMap';
import Node from './interfaces/Node';
import CNode from './interfaces/CNode';
export interface HandlePasteFile {
    (nodeKey: string, files: FileList): void;
}
export interface HandleAddNote {
    (nodeKey: string): void;
}
export interface HandleChangeNote {
    (nodeKey: string, note: string): void;
}
export interface HandleDeleteNote {
    (nodeKey: string): void;
}
export interface HandleDeleteAttach {
    (nodeKey: string, attachIndex: number): void;
}
interface HandleAddNext {
    (nodeKey: string, addedNode?: Node, value?: string): void;
}
interface TabAction {
    (nodeKey: string, targetKey: string, targetPlaceIndex?: number): void;
}
export interface HandleClickMore {
    (node: CNode, targetEl: HTMLElement): void;
}
interface HandleCommandChange {
    (nodeKey: string, command: string, value: string, addMode?: boolean): void;
}
export interface HandleClickUpload {
    (nodeKey: string, el: HTMLElement): void;
}
interface NodeClickFunc {
    (node: CNode, targetEl: HTMLElement): void;
}
interface MutiSelectFunc {
    (selectedNodes: Node[]): void;
}
export interface TreeEditorProps {
    nodes: NodeMap;
    startId: string;
    disabled?: boolean;
    readonly?: boolean;
    themeColor?: string;
    backgroundColor?: string;
    cutColor?: string;
    defaultFocusedId?: string;
    defaultNoteFocusedNodeId?: string;
    uncontrolled?: boolean;
    showIcon?: boolean;
    showPreviewButton?: boolean;
    quickCommandKey?: string;
    quickCommands?: string[];
    handleCommandChanged?: HandleCommandChange;
    handlePasteFiles: HandlePasteFile;
    handleDeleteAttach: HandleDeleteAttach;
    handleAddNote?: HandleAddNote;
    handleChangeNote?: HandleChangeNote;
    handleDeleteNote?: HandleDeleteNote;
    handleTabAction?: TabAction;
    handleClickExpand?: Function;
    handleClickNode?: Function;
    handleClickIcon?: Function;
    handleClickDot?: Function;
    handleChangeNodeText?: Function;
    handleAddNext?: HandleAddNext;
    handleAddPrevious?: HandleAddNext;
    handleAddChild?: Function;
    handleDeleteNode?: Function;
    handleShiftUpDown?: Function;
    handlePaste?: Function;
    handleDrag?: Function;
    handleClickMoreButton?: HandleClickMore;
    handleClickAddButton?: HandleClickMore;
    handleClickPreviewButton?: Function;
    handleClickUpload?: HandleClickUpload;
    handleCheck?: Function;
    handleClickAvatar?: NodeClickFunc;
    handleClickStatus?: NodeClickFunc;
    handleMutiSelect?: MutiSelectFunc;
    showDeleteConform?: Function;
    ref?: any;
    collapseMode?: boolean;
}
export declare const TreeEditor: React.ForwardRefExoticComponent<Pick<TreeEditorProps, "nodes" | "startId" | "disabled" | "readonly" | "themeColor" | "backgroundColor" | "defaultFocusedId" | "defaultNoteFocusedNodeId" | "uncontrolled" | "showIcon" | "showPreviewButton" | "quickCommandKey" | "quickCommands" | "handleCommandChanged" | "handlePasteFiles" | "handleDeleteAttach" | "handleAddNote" | "handleChangeNote" | "handleDeleteNote" | "handleTabAction" | "handleClickExpand" | "handleClickNode" | "handleClickIcon" | "handleClickDot" | "handleChangeNodeText" | "handleAddNext" | "handleAddPrevious" | "handleAddChild" | "handleDeleteNode" | "handleShiftUpDown" | "handleDrag" | "handleClickMoreButton" | "handleClickAddButton" | "handleClickPreviewButton" | "handleClickUpload" | "handleCheck" | "handleClickAvatar" | "handleClickStatus" | "handleMutiSelect" | "showDeleteConform" | "collapseMode" | "cutColor" | "handlePaste"> & React.RefAttributes<unknown>>;
export {};
