import Node from '../interfaces/Node';
import CNode from '../interfaces/CNode';
import NodeMap from '../interfaces/NodeMap';
import MutilSelectedNodeKey from '../interfaces/MutilSelectedNodeKey';
declare function findNodeById(nodes: CNode[], id: string): CNode | undefined;
declare function guid(len: number, radix: number): string;
declare function getShortedStr(str: string): string | undefined;
declare function textWidth(fontSize: number): {
    kanjiWidth: number;
    fullAngleWidth: number;
    halfAnglePunctuationWidth: number;
    alphabetWidth: number;
    numberWidth: number;
};
declare function textWidthAll(fontSize: number, text: string, bold?: boolean): number;
declare function splitTextIntoParagraphs(text: string, maxWidth: number, fontSize: number, bold?: boolean): string[];
declare function splitTextIntoParagraphsInTexts(texts: string[], textMaxWidth: number, fontSize: number, bold?: boolean): string[];
declare function getFullAngleNum(str: string): number;
declare function getHalfAnglePunctuationNum(str: string): number;
declare function getAlphabetNum(str: string): number;
declare function getNumberNum(str: string): number;
declare function getTextWidth(text: string, textMaxWidth: number, fontSize: number, bold?: boolean): {
    width: number;
    texts: string[];
};
declare function getNodeWidth(node: Node, fontSize: number, showIcon: boolean, showAvatar: boolean, avatarRadius: number, showChildNum: boolean, textMaxWidth: number, inputNodeKey?: string): number;
declare function getExtInfoWidth(node: Node, showIcon: boolean, showAvatar: boolean, avatarRadius: number, showChildNum: boolean): number;
declare function nodeLocation(node: CNode, type: string, BLOCK_HEIGHT: number, showIcon: boolean, showAvatar: boolean, avatarRadius: number, showChildNum: boolean): {
    x: number;
    y: number;
    emojiY?: undefined;
} | {
    x: number;
    y: number;
    emojiY: number;
} | null;
declare const isDragValid: (dragId: string, dropId: string, nodeMap: NodeMap) => boolean | undefined;
declare const isMutilDragValid: (dragIds: any[], dropId: string, nodeMap: NodeMap) => boolean;
/**
 * 获取有效的多选节点（框选拖拽节点时，如果同时选择父节点与其子节点，只有父节点为有效节点）
 * @param selectedNodes
 * @param nodeMap
 */
declare const getValidSelectedNodes: (selectedNodes: Node[], nodeMap: NodeMap) => MutilSelectedNodeKey[];
export declare const getNextSelect: (node: Node, nodeMap: NodeMap) => string | null;
export declare const getAncestor: (node: Node, nodeMap: NodeMap, includeSelf?: boolean | undefined) => string[];
export declare const getDescendantKeys: (nodeKeys: string[], nodeMap: NodeMap) => string[];
declare function addChildNode(nodeMap: NodeMap, selectedId: string, value?: string): {
    nodes: {
        [x: string]: Node;
    };
    addedNode: Node;
};
declare function addNextNode(nodeMap: NodeMap, selectedId: string, value?: string, isPrevious?: boolean): {
    nodes: {
        [x: string]: Node;
    };
    addedNode: Node;
};
declare function changeSortList(nodeMap: NodeMap, selectedId: string, type: 'up' | 'down'): {
    nodes: NodeMap;
    brotherKeys: string[];
} | null;
declare function deleteNode(nodeMap: NodeMap, selectedId: string): {
    nodes: {
        [x: string]: Node;
    };
    nextSelectNodeKey: string;
};
declare function toFatherBrother(nodeMap: NodeMap, nodeId: string): {
    nodes: {
        [x: string]: Node;
    };
};
declare function toBrotherChild(nodeMap: NodeMap, nodeIndex: number, nodeId: string, brotherId: string): {
    nodes: {
        [x: string]: Node;
    };
};
declare function dot(nodeMap: NodeMap, nodeId: string): {
    [x: string]: Node;
};
declare function checkNode(nodeMap: NodeMap, nodeId: string): {
    [x: string]: Node;
};
declare function changeNodeText(nodeMap: NodeMap, id: string, text: string): {
    [x: string]: Node;
};
declare function updateNodeByKey(nodeMap: NodeMap, id: string, data: any): {
    [x: string]: Node;
};
declare function addNodeNote(nodeMap: NodeMap, nodeId: string): {
    nodes: {
        [x: string]: Node;
    };
};
declare function deleteNodeNote(nodeMap: NodeMap, nodeId: string): {
    nodes: {
        [x: string]: Node;
    };
};
declare function changeNodeNote(nodeMap: NodeMap, id: string, text: string): {
    [x: string]: Node;
};
declare function editNode(c_nodes: Node[], nodeId: string, prop: any): Node[];
declare function save(c_nodes: Node[]): any;
declare function pasteNode(map: NodeMap, pasteNodeKey: string, pasteType: string, targetNodeKey: string): {
    [x: string]: Node;
} | null | undefined;
declare function copyNode(nodeMap: NodeMap, nodeKey: string): void;
declare function dragSort(map: NodeMap, dragId: string, dropId: string, placement: 'in' | 'up' | 'down'): {
    [x: string]: Node;
} | null;
declare function getNodesInSelection(selectionX: number, selectionY: number, selectionWidth: number, selectionHeight: number, nodeHeight: number, cnodes: Node[]): Node[];
declare function changeSelect(selectedId: string, cnodes: Node[], direction: 'ArrowUp' | 'ArrowRight' | 'ArrowDown' | 'ArrowLeft'): Node | undefined;
declare function changeMindSelect(selectedId: string, cnodes: Node[], direction: 'ArrowUp' | 'ArrowRight' | 'ArrowDown' | 'ArrowLeft'): Node | undefined;
declare function moveCursorToEnd(elment: HTMLDivElement): void;
declare function getTextAfterCursor(elment: HTMLDivElement): string;
declare function moveCursor(type: 'up' | 'down', cnodes: Node[], nodeId: string): Node | undefined;
declare function isCursorHead(): boolean | "";
declare function isCursorTail(elment: HTMLDivElement): boolean;
declare function getCursorIndex(): number | undefined;
declare function isMobile(): boolean;
declare function mouseDirection(element: Element, e: MouseEvent): string;
declare function getElPosition(el: HTMLElement): {
    x: number;
    y: number;
};
declare function isEmoji(str?: string): boolean;
declare function countNodeDescendants(nodeMap: NodeMap, nodeId: string): number;
declare const urlReg: RegExp;
declare const rainbowColors: string[];
declare const lightRainbowColors: {
    '#fa6761': string;
    '#f7b370': string;
    '#f5db4d': string;
    '#32c995': string;
    '#6c87ff': string;
    '#706dcb': string;
};
export declare function createObjectUrl(content: string[], type: string): string;
export declare function download(url: string, fileName?: string): void;
export declare function downloadSvgAsPng(svg: string, width: number, height: number, title: string): Promise<unknown>;
export declare function getImgData(imgUrl: string): Promise<unknown>;
export declare function downloadSvgAsPdf(svg: string, width: number, height: number, title: string): Promise<unknown>;
export { findNodeById, textWidth, textWidthAll, getNodeWidth, getShortedStr, getExtInfoWidth, nodeLocation, changeNodeText, addChildNode, addNextNode, deleteNode, dot, checkNode, editNode, save, dragSort, changeSortList, pasteNode, getFullAngleNum, getHalfAnglePunctuationNum, getAlphabetNum, getNumberNum, guid, changeSelect, changeMindSelect, getNodesInSelection, getValidSelectedNodes, isDragValid, isMutilDragValid, addNodeNote, moveCursorToEnd, changeNodeNote, deleteNodeNote, getTextAfterCursor, toBrotherChild, moveCursor, isCursorHead, toFatherBrother, isCursorTail, getCursorIndex, isMobile, mouseDirection, getElPosition, isEmoji, urlReg, updateNodeByKey, countNodeDescendants, splitTextIntoParagraphs, splitTextIntoParagraphsInTexts, getTextWidth, copyNode, rainbowColors, lightRainbowColors, };
